/**
   EpgScheduler - Automatic schedule of WinTV recordings via tvtv.de EPG
   Copyright (C) 2002 Steffen Siebert <siebert@SteffenSiebert.de>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package de.steffensiebert.epgscheduler;

import com.jobjects.jst.*;

import de.steffensiebert.epgscheduler.xml.*;
import de.steffensiebert.jwindowsscheduler.*;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.lang.NumberFormatException;
import java.text.MessageFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.Vector;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.WriterAppender;

import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.Unmarshaller;
import org.exolab.castor.xml.ValidationException;

public class EpgScheduler {
  private final static String VERSION_STRING = "EpgScheduler 1.2";

  private final static int SYNCIT_REGISTRY_TOP_KEY = RegistryKey.KEY_LOCAL_MACHINE;
  private final static String SYNCIT_REGISTRY_NAME = "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\App Paths\\SyncIT.exe";
  private final static String SYNCIT_REGISTRY_PATH_KEY = "Path";

  private final static int SYNCIT_REGISTRY_TVTV_TOP_KEY = RegistryKey.KEY_CURRENT_USER;
  private final static String SYNCIT_REGISTRY_TVTV_NAME = "Software\\SyncIT\\tvtv.de";
  private final static String SYNCIT_REGISTRY_TVTV_EMAIL_KEY = "email";
  private final static String SYNCIT_REGISTRY_TVTV_LOGIN_KEY = "login";

  private final static int WINTV_REGISTRY_TOP_KEY = RegistryKey.KEY_LOCAL_MACHINE;
  private final static String WINTV_REGISTRY_NAME = "SOFTWARE\\hauppauge\\Radio";
  private final static String WINTV_REGISTRY_PATH_KEY = "TVAppPath";
  private final static String WINTV_REGISTRY_NAME2 = "SOFTWARE\\hauppauge\\WinTV32\\CODECS\\StreamDvcrSolo";
  private final static String WINTV_REGISTRY_PATH2_KEY = "StreamDvcrSoloRecordFilePath";

  private final static String EPG_COMMENT_MARKER = "HCW Scheduler fs:";
  private final static String EPG_CLEANUP_COMMENT_MARKER = "HCW Scheduler Cleanup fs:";

  private final static String CONSOLE_LOGGING_PATTERN = "%d{HH:mm:ss} - %-5p %m%n";
  private final static String FILE_LOGGING_PATTERN = "%d{yy-MM-dd HH:mm:ss} - %-5p (%c{1}) %m%n";

  private final static String DEFAULT_RECORDING_PARAMETERS = "{0} -ntod -startr:{1}.mpg -qvcd -limit:{2,number,0}";

  private static Logger logger = Logger.getLogger(EpgScheduler.class);

  private static WindowsScheduler windowsScheduler = null;
  private static HashMap channels = null;
  private static String syncitPath = null;
  private static String tvtvLogin = null;
  private static String tvtvEmail = null;
  private static String wintvPath = null;
  private static String wintvOutputPath = null;
  private static String playlistPath = null;
  private static String playlistFileName = null;
  private static File playlistFile = null;
  private static String accountName = null;
  private static int defaultStartMinutes = 0;
  private static int defaultEndMinutes = 0;
  private static boolean mute = false;
  private static Vector taskNames = null;
  private static Vector outputFiles = null;
  private static ResourceBundle resourceBundle = null;
  private static boolean deleteSchedules = false;
  private static boolean createCleanupTask = false;
  private static String cleanupTaskCommand = null;
  private static String cleanupTaskParameter = null;
  private static boolean useWakeupFlag = false;
  private static String recordingApplication = null;
  private static String recordingParameters = DEFAULT_RECORDING_PARAMETERS;

  private static class OutputFilesFilter implements FilenameFilter {
    public boolean accept(File file, String name) {
      if (name.endsWith(".txt") || name.endsWith(".mpg")) {
        return true;
      } else {
        return false;
      }
    }
  }

  private static class TaskName {
    private String base = null;
    private int number = 0;

    TaskName(String base) {
      this.base = base;
      this.number = 0;
    }

    void inc() {
      number++;
    }

    public String toString() {
      if (number == 0) {
        return base;
      } else {
        return base+String.valueOf(number);
      }
    }
  }

  private static HashMap readChannels(BufferedReader in) throws IOException {
    HashMap channels = new HashMap();
    String line = null;
    while ((line = in.readLine()) != null) {
      String channel = null;
      String channelName = null;
      int beginMinutes = -1;
      int endMinutes = -1;

      int pos = line.indexOf(',');
      if (pos == -1) {
        logger.error("Korrupte Zeile in dbchannel.dat: "+line);
        continue;
      }
      channel = line.substring(0,pos);
      try {
        Integer.parseInt(channel);
        channel = "-c"+channel;
      } catch (NumberFormatException e) {
        // Keine reine Kanalnummer -> Kommandozeilenoption ist bereits enthalten
      }
      line = line.substring(pos+1);
      
      pos = line.indexOf(',');
      if (pos == -1) {
        channelName = line;
        beginMinutes = -1;
        endMinutes = -1;
      } else {
        channelName = line.substring(0,pos);
        line = line.substring(pos+1);
        pos = line.indexOf(',');
        if (pos == -1) {
          beginMinutes = Integer.parseInt(line);
          endMinutes = -1;
        } else {
          beginMinutes = Integer.parseInt(line.substring(0,pos));
          endMinutes = Integer.parseInt(line.substring(pos+1));
        }
      }
      channels.put(channelName, new Object[] {channel, new Integer(beginMinutes), new Integer(endMinutes)});
    }
    return channels;
  }

  private static String format(String text) {
    int lineLen = 0;
    int ws = 0;
    StringBuffer buffer = new StringBuffer();
    StringBuffer line = new StringBuffer();
    char[] array = text.toCharArray();
    for (int i=0; i<array.length; i++) {
      char c = array[i];
      if ('\n' == c) {
        line.append('\r');
        line.append(c);
        buffer.append(line.toString());
        line = new StringBuffer();
        ws = 0;
        lineLen = 0;
        continue;
      }
      if (' ' == c) {
        if (lineLen > 75) {
          line.append('\r');
          line.append('\n');
          buffer.append(line.toString());
          line = new StringBuffer();
          ws = 0;
          lineLen = 0;
          continue;
        } else {
          line.append(c);
          ws = lineLen;
          lineLen++;
          continue;
        }
      }
      if (lineLen > 79) {
        buffer.append(line.substring(0,ws));
        buffer.append('\r');
        buffer.append('\n');
        line = new StringBuffer(line.substring(ws+1));
        ws = 0;
        lineLen = line.length();
      }
      line.append(c);
      lineLen++;
    }
    buffer.append(line.toString());
    return buffer.toString();
  }

  private static boolean taskNameExist(Vector taskNames, TaskName taskName) {
    String name = taskName.toString()+".job";
    boolean found = false;
    for (int i=0; i<taskNames.size(); i++) {
      if (name.equals((String) taskNames.elementAt(i))) {
        found = true;
        break;
      }
    }
    return found;
  }

  private static boolean fileNameExist(Vector fileNames, TaskName taskName) {
    String name = taskName.toString();
    boolean found = false;
    for (int i=0; i<fileNames.size(); i++) {
      String file = (String) fileNames.elementAt(i);
      if (file.equals(name+".txt") || file.equals(name+".mpg")) {
        found = true;
        break;
      }
    }
    return found;
  }

  private static WindowsTask createSchedule(Epg_schedule_entry entry, int globalStartMinutes, int globalEndMinutes, String wintvPath) {
    SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    Date start = null;
    Date end = null;

    try {
      start = f.parse(entry.getStarttime());
      end = f.parse(entry.getEndtime());
    } catch (ParseException e) {
      logger.error("Fehler beim Parsen der Start/Endzeit", e);
      return null;
    }

    GregorianCalendar startCalendar = new GregorianCalendar();
    startCalendar.setTime(start);
    GregorianCalendar endCalendar = new GregorianCalendar();
    endCalendar.setTime(end);

    Object[] channel = (Object[]) channels.get(entry.getChannel());

    if (channel == null) {
      logger.warn("Channel "+entry.getChannel()+" nicht gefunden!");
      return null;
    }

    String channelString = ((String) channel[0]);
    int startMinutes = ((Integer) channel[1]).intValue();
    int endMinutes = ((Integer) channel[2]).intValue();

    if (mute) {
      channelString = channelString + " -mute";
    }

    if (startMinutes == -1) {
      startMinutes = globalStartMinutes;
    }

    if (endMinutes == -1) {
      endMinutes = globalEndMinutes;
    }

    startCalendar.add(GregorianCalendar.MINUTE,-1*startMinutes);
    endCalendar.add(GregorianCalendar.MINUTE,endMinutes);

    Date now = new Date();
    if (now.after(endCalendar.getTime())) {
      logger.info("Die Aufnahme liegt in der Vergangenheit!");
      return null;
    }
    if (now.after(startCalendar.getTime())) {
      logger.info("Die Sendung luft bereits, starte die Aufnahme in einer Minute!");
      startCalendar.setTime(now);
      startCalendar.add(GregorianCalendar.MINUTE,1);
    }

    long runtime = endCalendar.getTime().getTime() - startCalendar.getTime().getTime();

    WindowsTask t = new WindowsTask();
    String title = entry.getTitle().getContent();
    String name = title.replace(' ','_');
    StringBuffer namebuffer = new StringBuffer(name.length());
    for (int i=0; i<name.length(); i++) {
      char c = name.charAt(i);
      if ((c == '_') || ((c >= '0') && (c <= '9')) || ((c >= 'a') && (c <= 'z')) || ((c >= 'A') && (c <= 'Z'))) {
        namebuffer.append(c);
      }
    }
    TaskName taskName = new TaskName(namebuffer.toString());
    // Testen, ob ein Task oder eine Datei mit gleichem Namen existiert, falls ja
    // Namen um Zahl erweitern, bis Name noch nicht existiert.
    while (taskNameExist(taskNames, taskName) || fileNameExist(outputFiles, taskName)) {
      taskName.inc();
    }
    name = taskName.toString();
    t.setTaskName(name);
    if (recordingApplication == null || recordingApplication.trim().equals("")) {
      t.setApplication(wintvPath+"WinTV2K.EXE");
    } else {
      t.setApplication(recordingApplication);
    }
    Object[] arguments = {channelString, name, new Long(runtime/1000)};
    String parameter = MessageFormat.format(recordingParameters, arguments);
    t.setParameter(parameter);
    t.setWorkingDirectory("");
    t.setPriority(32);
    t.setComment(EPG_COMMENT_MARKER+entry.getUid());
    t.setRunTime(-1);
    int flags = WindowsTask.TASK_FLAG_DELETE_WHEN_DONE + WindowsTask.TASK_FLAG_RUN_ONLY_IF_LOGGED_ON;
    if (useWakeupFlag) {
      flags = flags + WindowsTask.TASK_FLAG_SYSTEM_REQUIRED;
    }
    t.setFlags(flags);
    String betriebssystem = System.getProperty("os.name");
    if ("Windows 95".equals(betriebssystem) || "Windows 98".equals(betriebssystem) || "Windows Me".equals(betriebssystem)) {
      logger.info("Windows 9x Betriebssystem -> Setze keine Account Informationen");
    } else {
      t.setAccountName(accountName);
      t.setPassword(null);
    }
    windowsScheduler.writeTask(t);
    WindowsTaskTrigger tr = new EpgTriggerOnce(startCalendar, endCalendar);

    logger.info("Erzeuge Task: "+entry.getChannel()+" - "+name);
    logger.info(tr);

    t.writeTrigger(tr);

    if (createCleanupTask && (cleanupTaskCommand != null)) {
      WindowsTask ct = new WindowsTask();
      ct.setTaskName(name+" Cleanup");
      ct.setApplication(cleanupTaskCommand);
      if (cleanupTaskParameter != null) {
        ct.setParameter(cleanupTaskParameter);
      } else {
        ct.setParameter("");
      }
      ct.setWorkingDirectory("");
      ct.setPriority(32);
      ct.setComment(EPG_CLEANUP_COMMENT_MARKER+entry.getUid());
      ct.setRunTime(-1);
      ct.setFlags(WindowsTask.TASK_FLAG_DELETE_WHEN_DONE+WindowsTask.TASK_FLAG_RUN_ONLY_IF_LOGGED_ON);
      if ("Windows 95".equals(betriebssystem) || "Windows 98".equals(betriebssystem) || "Windows Me".equals(betriebssystem)) {
        logger.info("Windows 9x Betriebssystem -> Setze keine Account Informationen");
      } else {
        ct.setAccountName(accountName);
        ct.setPassword(null);
      }
      windowsScheduler.writeTask(ct);
      // Cleanup Script wird eine Minute nach Ende der Aufnahme gestartet
      endCalendar.add(Calendar.MINUTE, 1);
      WindowsTaskTrigger ctr = new WindowsTaskTriggerOnce(endCalendar.get(Calendar.DAY_OF_MONTH),
                                                          endCalendar.get(Calendar.MONTH)+1,
                                                          endCalendar.get(Calendar.YEAR),
                                                          endCalendar.get(Calendar.HOUR_OF_DAY),
                                                          endCalendar.get(Calendar.MINUTE));
      logger.info("Erzeuge Cleanup Task: "+name+" Cleanup");
      ct.writeTrigger(ctr);
    }

    if (wintvOutputPath == null) {
      return t;
    }

    String descriptionFile = wintvOutputPath + name + ".txt";
    FileWriter writer = null;

    if (entry.getDescription() != null) {
      String description = format(entry.getDescription().getContent());
      try {
        writer = new FileWriter(descriptionFile);
        writer.write(description,0,description.length());
      } catch (IOException e) {
        logger.error("Fehler beim Erzeugen des Beschreibungsfiles", e);
      } finally {
        if (writer != null) {
          try {
            writer.close();
          } catch (IOException ignore) {
            // ignore
          }
        }
      }
    }
    return t;
  }

  private static String getRegistryKeyValue(int topKey, String subKey, String propName) {
    String value = null;
    if(!SystemToolkit.doesSupportRegistry()) {
      return null;
    }
    try {
      RegistryManager regm = SystemToolkit.getRegistryManager();
      RegistryKey key = regm.openRegistryKey(topKey, subKey);
      value = key.getStringProperty(propName);
    } catch(RegistryException e) {
      logger.error("Kann Registry Key "+subKey+" "+propName+" nicht lesen!", e);
    }
    return value;
  }

  private static HashMap getEpgTasks(Vector taskNames, String commentMarker) {
    HashMap tasks = new HashMap();
    for (int i = 0; i < taskNames.size(); i++) {
      String taskName = (String) taskNames.elementAt(i);
      WindowsTask task = windowsScheduler.readTask(taskName);
      String comment = task.getComment();

      logger.debug("Task Kommentar: "+comment);

      if (task.getComment().startsWith(commentMarker)) {
        String uid = comment.substring(commentMarker.length());

        logger.debug("EPG Task uid: "+uid);

        tasks.put(uid,task);
      }
    }
    return tasks;
  }

  private static void work(boolean initialRun) {
    FileReader reader = null;
    Epg_schedule schedule = null;

    logger.info("Parse XML-File");

    try {
      playlistFile = new File(playlistFileName);
      reader = new FileReader(playlistFile);
      schedule = (Epg_schedule) Unmarshaller.unmarshal(Epg_schedule.class, reader);
    } catch (IOException e) {
      logger.error("IOException in work()",e);
      return;
    } catch (MarshalException e) {
      if (initialRun) {
        logger.warn("MarshalException in initial run of work()");
        logger.warn("Assuming that no internet connection is available.");
      } else {
        logger.error("MarshalException in work()",e);
      }
      return;
    } catch (ValidationException e) {
      logger.error("ValidationException in work()",e);
      return;
    } finally {
      if (reader != null) {
        try {
          reader.close();
        } catch (IOException ignore) {
          // Ignore
        }
      }
    }

    Epg_schedule_entry[] entries = schedule.getEpg_schedule_entry();

    taskNames = windowsScheduler.getTasks();
    HashMap tasks = getEpgTasks(taskNames, EPG_COMMENT_MARKER);
    HashMap oldTasks = (HashMap) tasks.clone();
    HashMap cleanupTasks = getEpgTasks(taskNames, EPG_CLEANUP_COMMENT_MARKER);

    // Alle bereits existierenden .mpg und .txt Dateien ermitteln
    File wintvOutput = new File(wintvOutputPath);
    outputFiles = new Vector();
    String[] outputFilesArray = wintvOutput.list(new OutputFilesFilter());
    for (int i=0; i<outputFilesArray.length; i++) {
      outputFiles.add(outputFilesArray[i]);
    }

    for (int i=0; i<entries.length; i++) {
      Epg_schedule_entry entry = entries[i];

      logger.info("Task: "+entry.getTitle().getContent());
      logger.debug("UID: "+entry.getUid());
      logger.debug("Channel: "+entry.getChannel());

      String uid = entry.getUid();
      WindowsTask task = (WindowsTask) tasks.get(uid);
      if (task != null) {
        logger.info("Der Task existiert bereits: "+task.getTaskName());
        oldTasks.remove(uid);
      } else {
        try {
          WindowsTask newTask = createSchedule(entry, defaultStartMinutes, defaultEndMinutes, wintvPath);
          if (newTask != null) {
            tasks.put(uid,newTask);
            taskNames.add(newTask.getTaskName()+".job");
          }
        } catch (OleTaskExistException e) {
          logger.error("Fehler beim Erzeugen des Task",e);
        }
      }
    }
    
    if (deleteSchedules) {
      // Alle alten Tasks, die nicht im aktuellen XML-File sind, werden geloescht
      if (!oldTasks.isEmpty()) {
        for (Iterator i = oldTasks.keySet().iterator(); i.hasNext();) {
          String uid = (String) i.next();
          WindowsTask task = (WindowsTask) oldTasks.get(uid);
          logger.info("Loesche Task: "+task.getTaskName());
          windowsScheduler.deleteTask(task);
          // Falls ein zugehoeriger Cleanup Task existiert, diesen auch loeschen
          if (cleanupTasks.containsKey(uid)) {
            task = (WindowsTask) cleanupTasks.get(uid);
            logger.info("Loesche Cleanup-Task: "+task.getTaskName());
            windowsScheduler.deleteTask(task);
          }
        }
      }
    }
  }

  private static void run() {
    wintvPath = getRegistryKeyValue(WINTV_REGISTRY_TOP_KEY, WINTV_REGISTRY_NAME, WINTV_REGISTRY_PATH_KEY);
    if (wintvPath == null) {
      logger.fatal("Kann den Pfad von WinTV2K in der Registry nicht finden!");
      return;
    }

    logger.info("WinTV2k Path: " + wintvPath);

    wintvOutputPath = getRegistryKeyValue(WINTV_REGISTRY_TOP_KEY, WINTV_REGISTRY_NAME2, WINTV_REGISTRY_PATH2_KEY);
    if (wintvOutputPath == null) {
      logger.error("Kann den Output-Pfad von WinTV2K in der Registry nicht finden!");
    } else {
      logger.info("WinTV2k Output-Path: " + wintvOutputPath);
    }

    syncitPath = getRegistryKeyValue(SYNCIT_REGISTRY_TOP_KEY, SYNCIT_REGISTRY_NAME, SYNCIT_REGISTRY_PATH_KEY);
    tvtvLogin = getRegistryKeyValue(SYNCIT_REGISTRY_TVTV_TOP_KEY, SYNCIT_REGISTRY_TVTV_NAME, SYNCIT_REGISTRY_TVTV_LOGIN_KEY);
    tvtvEmail = getRegistryKeyValue(SYNCIT_REGISTRY_TVTV_TOP_KEY, SYNCIT_REGISTRY_TVTV_NAME, SYNCIT_REGISTRY_TVTV_EMAIL_KEY);

    if (syncitPath == null) {
      logger.fatal("Kann den Pfad von SyncIt in der Registry nicht finden!");
      return;
    }

    if (tvtvLogin == null) {
      logger.fatal("Kann den Login-Namen fr SyncIt in der Registry nicht finden!");
      return;
    }
      
    if (tvtvEmail == null) {
      logger.fatal("Kann die Email-Adresse fr SyncIt in der Registry nicht finden!");
      return;
    }

    playlistPath = syncitPath + "\\" + tvtvEmail + "-" + tvtvLogin + "\\" + "cache";
    playlistFileName = playlistPath + "\\" + "playlist.xml";

    logger.info("Playlist File: " + playlistFileName);

    accountName = System.getProperty("user.name");

    logger.info("Account Name: " + accountName);

    try {
      FileReader reader = new FileReader("dbchannel.dat");
      channels = readChannels(new BufferedReader(reader));
      reader.close();
    } catch (IOException e) {
      logger.fatal("Fehler beim Lesen von dbchannel.dat", e);
      return;
    }

    Object[] channel = (Object[]) channels.get("<default>");
    if (channel == null) {
      logger.warn("channel <default> not found!");
    } else {
      defaultStartMinutes = ((Integer) channel[1]).intValue();
      defaultEndMinutes = ((Integer) channel[2]).intValue();
    }

    logger.info("DefaultStartMinutes: "+defaultStartMinutes);
    logger.info("DefaultEndMinutes: "+defaultEndMinutes);

    channel = (Object[]) channels.get("<mute>");
    if (channel == null) {
      logger.warn("channel <mute> not found!");
    } else {
      if ("n".equals((String) channel[0])) {
        mute = false;
      } else if ("y".equals((String) channel[0])) {
        logger.warn("WinTV wird bei der Aufnahme stummgeschaltet");
        mute = true;
      } else {
        logger.warn("Illegaler Wert "+channel[0]+" fuer Kanal <mute>.");
      }
    }

    windowsScheduler = new WindowsScheduler();

    work(true);

    while (true) {
      Ole.WaitForFileChange(playlistPath);
      // FIXME
      // Wir mssen hier warten, weil SyncIt zuerst ein temporres File
      // erzeugt und dann dieses umbenennt. Wenn wir das XML File zu frh
      // lesen, dann hat es noch den alten Inhalt und SyncIt kann das
      // temporre File nicht umbenennen, wie das XML File durch uns
      // geffnet ist.
      try {
        Thread.sleep(1000);
      } catch (InterruptedException ignore) {
        // Ignore
      }
      work(false);
    }
  }

  public static void main(String argc[]) {
    // Logging initialisieren
    Logger root = Logger.getRootLogger();
    PatternLayout consoleLayout = new PatternLayout(CONSOLE_LOGGING_PATTERN);
    PatternLayout fileLayout = new PatternLayout(FILE_LOGGING_PATTERN);
    ConsoleAppender console = new ConsoleAppender(consoleLayout);
    console.setThreshold((Level) Level.INFO);
    root.addAppender(console);
    try {
      WriterAppender writer = new WriterAppender(fileLayout, new FileWriter("epgsched.log", true)); 
      root.addAppender(writer);
    } catch (IOException e) {
      logger.error("Fehler beim ffnen des Logfiles.",e);
    }

    logger.info(VERSION_STRING);
    logger.info("Start");
    logger.info("Betriebssystem: "+System.getProperty("os.name"));

    try {
      resourceBundle = ResourceBundle.getBundle("epgscheduler");
    } catch (MissingResourceException e) {
      logger.error("Kein ResourceBundle epgscheduler gefunden.",e);
    }

    if (resourceBundle != null) {
      try {
        deleteSchedules = new Boolean(resourceBundle.getString("deleteSchedules").trim()).booleanValue();
      } catch (MissingResourceException e) {
        logger.error("Keine Property deleteSchedules gefunden.");
      }
    }
    logger.info("deleteSchedules: "+deleteSchedules);
    
    if (resourceBundle != null) {
      try {
        createCleanupTask = new Boolean(resourceBundle.getString("createCleanupTask").trim()).booleanValue();
      } catch (MissingResourceException e) {
        logger.error("Keine Property createCleanupTask gefunden.");
      }
    }
    logger.info("createCleanupTask: "+createCleanupTask);
    
    if (createCleanupTask) {
      if (resourceBundle != null) {
        try {
          cleanupTaskCommand = resourceBundle.getString("cleanupTaskCommand");
        } catch (MissingResourceException e) {
          logger.error("Keine Property cleanupTaskCommand gefunden.");
        }
      }
      if (cleanupTaskCommand != null) {
        File commandFile = new File(cleanupTaskCommand);
        if (commandFile.exists()) {
          if (!commandFile.isAbsolute()) {
            cleanupTaskCommand = commandFile.getAbsolutePath();
          }
          logger.info("cleanupTaskCommand: "+cleanupTaskCommand);
        } else {
          logger.error("Die cleanupTaskCommand Datei "+cleanupTaskCommand+" existiert nicht!");
          cleanupTaskCommand = null;
        }
      }
      if (resourceBundle != null) {
        try {
          cleanupTaskParameter = resourceBundle.getString("cleanupTaskParameter");
        } catch (MissingResourceException e) {
          logger.error("Keine Property cleanupTaskParameter gefunden.");
        }
      }
      logger.info("cleanupTaskParameter: "+cleanupTaskParameter);
    }
    
    if (resourceBundle != null) {
      try {
        useWakeupFlag = new Boolean(resourceBundle.getString("useWakeupFlag").trim()).booleanValue();
      } catch (MissingResourceException e) {
        logger.error("Keine Property useWakeupFlag gefunden.");
      }
    }
    logger.info("useWakeupFlag: "+useWakeupFlag);
    
    if (resourceBundle != null) {
      try {
        recordingApplication = resourceBundle.getString("recordingApplication");
        logger.info("recordingApplication: "+recordingApplication);
      } catch (MissingResourceException e) {
        logger.info("Keine Property recordingApplication gefunden.");
      }
    }
    
    if (resourceBundle != null) {
      try {
        recordingParameters = resourceBundle.getString("recordingParameters");
      } catch (MissingResourceException e) {
        logger.warn("Keine Property recordingParameters gefunden.");
      }
    }
    logger.info("recordingParameters: "+recordingParameters);
    
    try {
      run();
    } catch (Throwable throwable) {
      logger.fatal("Nicht behandelte Runtime-Exception", throwable);
    }

    logger.info("Ende");

  }
}
